% IMAGING Trac d'une diagraphie d'imagerie lectrique
%
% SYNTAXE 
%   fig=imaging(donnee,profdata,color, indices, mapcolor,file,orientation)
%
%	donnee	 : donnes  dessiner, sous formes
%      	                  d'une MATRICE DE size(donnee,2) COLONNES
% 	profdata : matrices des profondeurs, 1
%     	                  colonnes, de longueur quelconques 
%	color	 : vecteur 2*1, donnant les bornes de
%	                  l'intervalle de couleurs 
%			   avec lequel dessiner. Pour une slection
%			   judicieuse,voir 'selectrange.m' 
%	indices	 :  vecteur 2*1 donnant les intervalles des blocs de
%	                 'donnes'  dessiner 
%                       Pour une slection judicieuse, voir 'selectprof.m' 
%	mapcolor : colormap de la figure. donc matrice N*3,
%	                  o N vaut souvent 64; 
%			  Pour les colormaps 'standards' (ex. jet,
%     	                  hsv), utiliser la syntaxe :
%                                        mapcolor=jet(64)
%     	                              ou mapcolor=hsv(64) 
%    			  Pour les colormaps 'maison' : 
%                         par exemple, 'bramanscolor',  
%				 	load bramanscolor ; 
%					colormap(map)    
%                                    ou mapcolor=map
%	titre	 :	titre de la diagraphie
%	fig	 :	numro des figures cres par ce script.
%       file     :   fichier a partir duquel la diagraphie a t cre
%                               avec l'aide de diagraphie.m
%       orientation : [lambda gamma]
%                      lambda : incidence en degrs
%                      gamma  : azimut en degrs
%                Utile pour avoir la position du forage pour la sinusode.
%
%	fig	 :	numro des figures cres par ce script.
%-------------------------------------------------------------------------
%
% Jouets interactifs :
%	- slection interactive du titre : au dessus des colonnes
% 	  de la diagraphie se trouve un grand rectangle. Si on clique dessus
%	  avec le bouton DROIT de la souris, on peut diter le
%	  titre, qui sera immdiatement rpercut sur toutes les
%	  figures de l'affichage
%	- slection interactive des lignes 
%               On peut ensuite effectuer quelques analyses sur les
%               blocs slectionns 
%                       => Diagrammes polaires
%                       => Fittage de sinusode
%
%

function fig=imaging(donnee,profdata,color, indices, mapcolor,titre,file,orientation);
if ~ischar(donnee)
    % Fonctionnement normal
    fullposition=[0 0.0365    1.0000    0.9141];
    % si il n'y a pas de figures auparavant, une fentre blanche et
    % inutile se formerait. 
    % On vrifi donc qu'il n'y en a pas avant de rgler le problme
    formerfigure=findobj('Type','figure');
    %
    % A - GESTION DES ARGUMENTS
       
    % A1 gestion de la profondeur
    % Si aucune profondeur n'est donne se contenter des indices
    if nargin<2
        profdata=[1:length(donnee)]';
    end
    
    %A2 gestion de l'indexage de couleur
    if nargin >= 3 & length(color)==2
        CMIN=min(color);
        CMAX=max(color);
    else
        disp('Indexage des couleurs par dfaut')
        % L'avantage de la syntaxe donnee(:) est qu'elle est
        % indpendante de la dimension de donnee 
        CMIN=min(donnee(:));
        CMAX=max(donnee(:));
    end      
 % On limine le cas o donnee est une matrice de dimension suprieure  2
    coef=size(colormap,1)/(CMAX-CMIN);
    datacol=coef*(donnee-CMIN*ones(size(donnee)));
   % Les valeurs de datacol sont redimensionnes pour centrer l'intervalle 
  % de couleurs voulu sur la colormap. Il est impossible de redimensionner
 % la colormap sur l'intervalle dsir : elle ne prend qu'un nombre entier 
    % de couleurs.
    
    %A3 gestion des indices affichs
    if nargin>3 & length(indices)==2 
        X1=min(indices); X2=max(indices);
    else 
        X1=1; X2=min([size(donnee, 1), size(profdata, 1)]);
    end
    nombreblocs=length(X1:X2);
    
    % A4 gestion de la colormap
    if (nargin>4) & (size(mapcolor,2)==3)
        map=mapcolor;
    else
        map=jet(64);
    end
    colormap(map);
    if isempty(formerfigure)
       close all;   % On limine la figure  inutile ventuellement forme.
    end
    
    % A5 Titre par dfaut
    if (nargin>5) & ischar(titre)
        pagetitle=titre;
    else 
        pagetitle='';
    end
    
    % matrices  afficher
    dataView=datacol(X1:X2,:);
    profView=profdata(X1:X2);
    dataref=donnee(X1:X2,:);   
    
    nombreblocs=length(X1:X2);
    nombrecolonnes=ceil(nombreblocs/250);  % colonnes de 250 blocs
    nombrepages=ceil(nombrecolonnes/5);    % 5 colonnes par pages
    lastcolnumber=mod(nombrecolonnes,5);
    if lastcolnumber==0
        lastcolnumber=5;
    end
    lastcollength=mod(nombreblocs,250);
    if lastcollength==0
        lastcollength=250;
    end
    fig=zeros(1,nombrepages);
    
    % B - TRACE DES FIGURES DE DIAGRAPHIE
    id=now;        % identificateur caractristique des figures que
    % l'on va tracer 
    % B1 trac des premires pages,
    for i=1:(nombrepages-1)
        % dfinition et mise en place de la figure
        open('squelette.fig');
        fig(i)=gcf;
        set(gcf,'userdata',id);
        set(findobj(gcf,'Tag','pagenumber'),...
            'string',[num2str(i) '/' num2str(nombrepages)]); 
        colormap(map);
        % cration du menu
        imaging('setmenu');
        set(findobj(fig(i),'Tag','pagetitle'),'string',pagetitle);
        % trac de la colorbar
        axes(findobj(gcf,'Type','axes','Tag','bottom'));
        set(gca,'NextPlot','add');
        set(gca,'Xlim', [1 length(colormap)],...
            'XTick', [1 length(colormap)/4 ...
            length(colormap)/2 0.75*length(colormap) length(colormap)],...
            'XTickLabel', [CMIN CMIN+(CMAX-CMIN)/4 ...
                CMIN+(CMAX-CMIN)/2 CMIN+3*(CMAX-CMIN)/4 CMAX]);
        image([1:length(colormap)]);
        % trace des diagraphie sur chacune des 5 colonnes
        for j=1:5
            AxNumber=(i-1)*5+j;
            axes(findobj(gcf,'Type','axes',...
                'Tag',['ax' num2str(j)]));
            set(gca,'NextPlot','add',...
                'Userdata',AxNumber);
            set(gca,'Xlim', [0.5 (size(donnee,2)+0.5)]);
            set(gca,'YLim', [1 250], 'YTick', [1:12:250],...
             'YTickLabel',profView(((AxNumber-1)*250+1):12:AxNumber*250));
            h=image(dataView( ((AxNumber-1)*250+1):(AxNumber*250),: ) );
            set(h,'ButtonDownFcn','imaginglines;');
        end
    end
    %B2 trac de la dernire page
    open('squelette.fig');
    fig(nombrepages)=gcf;
    set(gcf,'userdata',id);
    set(findobj(gcf,'Tag','pagenumber'),...
        'string',[num2str(nombrepages) '/' num2str(nombrepages)]);
    set(findobj(gcf,'Tag','pagetitle'),...
        'string', pagetitle);  
    colormap(map);
    imaging('setmenu');
    set(findobj(gcf,'Tag','pagetitle'),...
        'string',pagetitle);
    axes(findobj(gcf,'Type','axes','Tag','bottom'));
    set(gca,'Xlim', [1 length(colormap)],...
        'XTick', [1 length(colormap)/4 ...
            length(colormap)/2 0.75*length(colormap) length(colormap)],...
        'XTickLabel', [CMIN CMIN+(CMAX-CMIN)/4 ...
            CMIN+(CMAX-CMIN)/2 CMIN+3*(CMAX-CMIN)/4 CMAX]);
    h=image([1:length(colormap)]);
    set(h,'ButtonDownFcn','imaginglines;');
    % les premiers axes sont dessins normalement
    for j=1:(lastcolnumber-1)
        AxNumber=(nombrepages-1)*5+j;
        axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
        set(gca,'Xlim', [0.5 (size(donnee,2)+0.5)]);
        set(gca,'NextPlot','add','Userdata',AxNumber);
        set(gca,'YLim', [1 250], 'YTick', [1:12:250]);
     set(gca,'YTickLabel',profView(((AxNumber-1)*250+1):12:AxNumber*250));
        h=image(dataView(((AxNumber-1)*250+1):(AxNumber*250),:));
        set(h,'ButtonDownFcn','imaginglines;');
    end
    % On dessine le dernier axe
    axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(lastcolnumber)]));
    % La position de cet axe est le point critique
    % position=[Xmin Ymin XLarge YLarge]
    position=get(gca,'position');
    YLarge=ceil(100*lastcollength/250*position(4))/100;
    Ymin=position(2)+position(4)-YLarge;
    position(2)=Ymin; position(4)=YLarge;
    set(gca,'position',position);
    % La dfinition des axes est la dfinition usuelle
    set(gca,'Xlim',[0.5 (size(donnee,2)+0.5)]);
    set(gca,'NextPlot','add',...
        'Userdata',nombrecolonnes);
    %   set(gca,'Ydir','reverse','Xdir','normal','TickDir','out');
    set(gca,'YLim', [1 lastcollength],...
        'YTick', [1:12:lastcollength],...
       'YTickLabel', profView(((nombrecolonnes-1)*250+1):12:nombreblocs));
    h=image(dataView( ((nombrecolonnes-1)*250+1):nombreblocs,: ));
    set(h,'ButtonDownFcn','imaginglines');
    % On efface les axes en excs
    for j=(lastcolnumber+1):5
        delete(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
    end
    
    % C - Inscription des paramtres dans les figures
    %on sauvegarde  part les donnes dessines pour une rexploitation
    % dans polaire. On utilise pour cela les utilitaires Matlab/graphics
    %qui permettent d'viter de stocker des donnes dans des gros fichiers
    % que l'on oublie ensuite.
    setappdata(fig(1), 'data',donnee); %'polaire'(imaginglines));
    setappdata(fig(1),'depth',profdata); %'polaire'(imaginglines)
    setappdata(fig(1),'indices',[X1 X2]);
    setappdata(fig(1),'colormap',map);
    if nargin==8
        setappdata(fig(1),'orientation',orientation);
    end
    for i=1:length(fig)
        %     setappdata(gcf,'donnee',donnee); %'polaire'(imaginglines)
        %    setappdata(gcf,'profondeur',profdata);'polaire'(imaginglines)
        %     setappdata(gcf,'indices',[X1 X2]);
        %     setappdata(gcf,'colormap',map);
        setappdata(fig(i),'pages',fig); % 'saving', 'closing'
        setappdata(fig(i),'numero',i);  % 'printing', 'printing'
        setappdata(fig(i),'lineeditfig',{}); % 'closing'
        setappdata(fig(i),'lineid',{});   % 'closing'
        setappdata(fig(i),'id',id);
        setappdata(fig(i),'figdata',fig(1)); % Numero de la figure de
        % refrence qui stocke
        % les grosses donnes      
        set(fig(i),'CloseRequestFcn','imaging(''close'')',...
            'DeleteFcn','imaging(''delete'')')                                  
        if nargin>6
            setappdata(fig(i),'file',file)
        end
        if nargin>7
            set(findobj(fig(i),'Type','uimenu',...
                'Label','&Orientation'),...
                'Checked','on');
        end
    end
    % L'affichage se termine sur la premire page
    figure(fig(1));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%% CALLBACKS DE L'INTERFACE GRAPHIQUE %%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
else
    % Voici les callbacks des interfaces graphiques
    switch donnee
        % gestion du titre
    case 'text'
        set(gcbo,'Style','edit',...
            'Callback','imaging(''edit'')');
    case 'edit'
        set(gcbo,'Style','text',...
            'Callback','imaging(text)');
        samefig=findobj('Type','figure','Userdata',get(gcbf,'Userdata'));
        name=get(gcbo,'string');
        figureappel=gcf;
        for i=1:length(samefig)                         
        set(findobj(figure(samefig(i)),'Tag','pagetitle'),'string', name);
        end
        figure(figureappel);
        % fermeture des fentres
    case 'setmenu'
        colormenu;
        iddiagraphie=get(gcf,'Userdata');
        set(findobj(gcf,'Tag','diagraphiemenu'),'Userdata',iddiagraphie)
        set(findobj(gcf,'Type','uimenu','Label','imaging_log'),...
            'foregroundcolor',[0.5 0.5 0])   
        firstmenu=uimenu('Label','|<<','Userdata',iddiagraphie,...
            'Callback','imaging(''first'');','foregroundcolor',[1 0 0]);
        previousmenu=uimenu('Label','<<','Userdata',iddiagraphie,...
           'Callback','imaging(''previous'');','foregroundcolor',[1 0 0]);
        nextmenu=uimenu('Label','>>','Userdata',iddiagraphie,...
            'Callback','imaging(''next'');','foregroundcolor',[1 0 0]);
        lastmenu=uimenu('Label','>>|','Userdata',iddiagraphie,...
            'Callback','imaging(''last'')','foregroundcolor',[1 0 0]);
    case 'printing'
        % Sous-menu "Tout Imprimer"
        currentfigure=gcf; 
        iddiagraphie=get(gcbf,'Userdata');
        figures=findobj('Type','figure','Userdata',iddiagraphie);
       disp('L''impression de toutes les pages de la diagraphie commence')
        for i=1:length(figures);
            figure(figures(i));
            numero=getappdata(figures(i),'numero');
     disp(['impression de la page ' num2str(numero) ' de la diagraphie']);
            print -dwinc;
        end
        figure(currentfigure);
    case 'saving'
        % Sous-menu "Tout sauver"
        currentfigure=gcf;
     formatlist=str2mat('epsc','jpeg','bmp','mfig','mmat','fig','tiff',...
            'png','hdf','pcx','xwd');
        [format,ok]=listdlg('Name','Format ', ...
            'SelectionMode','single',....
            'Liststring',formatlist,...
            'ListSize',[100 140],...
            'fus',10);
        if ok
            format=formatlist(format,:);
        else
            return
        end
        name=inputdlg('saving name :','saving name');
        name=name{1};
        if isempty(name)
            return
        end
        % On obtient la liste des pages initialement cres.
        pages=getappdata(gcbf,'pages');
        % On vrifie que ces pages existent bien
        indices=find(ishandle(pages));
        pages=pages(indices);
        % On vrifie aussi qu'elles correspondent bien  la diagraphie
        idpages=get(pages,'Userdata');
        id=getappdata(gcbf,'id');
        % Idpages est de la structure cell si length(pages)>1
        if iscell(idpages)
            indices=find([idpages{:}]==id);
        else
            indices=find(idpages==id);
        end
        pages=pages(indices);
        % Barre d'attente pendant la sauvegarde
        h=waitbar(0,'Sauvegarde des figures...');
        nombre=length(pages);
        for i=1:nombre;
            figure(pages(i));
            numero=getappdata(gcf,'numero');
   %         namesave=[name num2str(numero) '.' format];
  %%%%%%%%%%impro fred
            name2fred=['-d' format];
            namesave=[name num2str(numero)];
  %%%%%%%%%%%%
            waitbar((i-1)/nombre,h);
   %         feval('saveas',gcf,namesave,format);
  %%%%%%%%%improfred suite
            feval('print',gcf,name2fred,namesave)
            waitbar(i/nombre,h);
            disp([ 'La page ' num2str(numero)...
                    ' de la diagraphie sauve sous le nom ' namesave]);         
        end
        close(h);
        disp(['Figures sauvegardes dans le rpertoire courant ' pwd]);
        figure(currentfigure);
    case 'closing'
        % Sous-menu "Tout fermer"
        % Attention : Matlab tendait  faire des segmentation faults 
     % Voila pourquoi ce bout de code peut sembler particulirement tordu.
        id=get(gcbf,'Userdata');
        fig=findobj('Type','figure','Userdata',id);
        for i=1:length(fig)
            set(fig(i),'CloseRequestFcn','closereq')
            close(fig(i));
        end
    case 'delete'
        % Proprit 'DeleteFcn' des figures de la diagraphie
        line=findobj(gcbf,'Type','line','Tag','debline');
        for j=1:length(line)
            idline=get(line(j),'Userdata');
            delete(findobj('Userdata',idline));
        end
    case 'close'
        % Proprit 'CloseRequestFcn' des figures de la diagraphie
        %  Ce qui est effectue quand on ferme la fonction
        % Les donnes volumineuses sont regroupes sur une seule page
        % On vrifie d'abord qu'on ne ferme pas cette page
        if isappdata(gcbf,'figdata')
            precaution=(gcbf==getappdata(gcbf,'figdata'));
        else
            precaution=boolean(0);
        end
        if precaution 
            %% ON RETROUVE LES PAGES CRES ET TOUJOURS EXISTANTES
            % On se souvient des pages cres
            pages=getappdata(gcbf,'pages');
            numeroini=1:length(pages);
            % On ne retient que les pages existantes ...
            indices=find(ishandle(pages));
            pages=pages(indices);
            numeroreste=numeroini(indices);
            % ... avec l'identificateur de la diagraphie (car de
            % nouvelles figures ont pu les remplacer) 
            id=get(gcbf,'Userdata');
            idpages=get(pages,'Userdata');
            if iscell(idpages)
                indices=find([idpages{:}]==id);
            else
                indices=find(idpages==id);
            end
            pages=pages(indices);
            numeroreste=numeroreste(indices);
            %% ON TRANSMET LES DONNES A UNE FIGURE SURVIVANTE
            if ~isempty(pages)
            % slection de la figure suivantes qui recueillera les donnes
                figdata=pages(numeroreste(1));
                if isappdata(gcbf,'donnee')
                    setappdata(figdata,'donnee',getappdata(gcbf,'donnee'))
                end
                if isappdata(gcbf,'profondeur')
            setappdata(figdata,'profondeur',getappdata(gcbf,'profondeur'))
                end
                if isappdata(gcbf,'indices')
                  setappdata(figdata,'indices',getappdata(gcbf,'indices'))
                end
                if isappdata(gcbf,'colormap')
                setappdata(figdata,'colormap',getappdata(gcbf,'colormap'))
                end         
                if isappdata(gcbf,'orientation')
          setappdata(figdata,'orientation',getappdata(gcbf,'orientation'))
                end
                for i=1:length(numeroreste)
                    setappdata(pages(numeroreste(i)),'figdata',figdata);
                end
            end
        end
        closereq
    case 'colormap'
        currentfigure=gcf;
        id=get(gcbf,'Userdata');
        pages=getappdata(gcbf,'pages');
  maps=str2mat('jet','hsv','bramanscolor','gray','bone','copper','pink',...
            'white','lines','colorcube','prism',...
            'autumn','spring','summer','winter'); 
        [choix,ok]=listdlg('Promptstring','Colormap : ',...
            'SelectionMode','single',...
            'Liststring',maps);
        if ~ok
            return
        end
        mapstring=deblank(maps(choix,:)); 
        for i=1:length(pages)
            figure(pages(i));
            try    
                feval('colormap', mapstring); 
            catch 
                try
                    feval('load',mapstring);
                    feval('colormap', map); 
                catch           
                    map=feval(mapstring,64); 
                    feval('colormap', map); 
                end 
            end   
        end
        figure(currentfigure);
    case 'orientation'
        current=gcf; 
        fig=diagraphieorientation;
        if isappdata(current,'orientation');
            orientation=getappdata(current,'orientation');
            gamma=orientation(:,2);
            lambda=orientation(:,1);
            setappdata(fig,'azimut',gamma);
            setappdata(fig,'incidence',lambda);
            if ndims(gamma)>1
                gamma=mean(gamma(:));
            end
            set(findobj(fig,'Tag','MeanAzimuth'),...
                'string',num2str(gamma));
            if ndims(lambda)>1
                lambda=mean(lambda(:));
            end
            set(findobj(fig,'Tag','MeanIncidence'),...
                'string',[num2str(lambda) '']);
            if isappdata(gcf,'file')
                setappdata(fig,'file',getappdata(gcf,'file'));
            end
        end;
        figure(fig);
        uiwait;
        if isappdata(fig,'orientation');
            orientation=getappdata(fig,'orientation');
      setappdata(getappdata(current,'figdata'),'orientation',orientation);
            set(gcbo,'Checked','on');
        end;
        delete('workspaceprovisoire.mat');  
        close(fig); 
    case 'first'
        pages=getappdata(gcbf,'pages');
        % On ne retient que les pages existantes ...
        indices=find(ishandle(pages));
        pages=pages(indices);
        % ... avec l'identificateur de la diagraphie
        id=get(gcbf,'Userdata');
        idpages=get(pages,'Userdata');
        if iscell(idpages)
            indices=find([idpages{:}]==id);
        else
            indices=find(idpages==id);
        end
        pages=pages(indices);
        % Aller 1e page
        figure(pages(1));
    case 'previous'
        pages=getappdata(gcbf,'pages');
        numeroini=1:length(pages);
        % On ne retient que les pages existantes ...
        indices=find(ishandle(pages));
        pages=pages(indices);
        numeroreste=numeroini(indices);
        % ... avec l'identificateur de la diagraphie
        id=get(gcbf,'Userdata');
        idpages=get(pages,'Userdata');
        if iscell(idpages)
            indices=find([idpages{:}]==id);
        else
            indices=find(idpages==id);
        end
        pages=pages(indices);
        numeroreste=numeroreste(indices);
        % Aller page suivante
        numero=getappdata(gcbf,'numero');
        numero=find(numeroreste==numero);
        if numero~=1
            figure(pages(numero-1));
        end
    case 'next'
        pages=getappdata(gcbf,'pages');
        numeroini=1:length(pages);
        % On ne retient que les pages existantes ...
        indices=find(ishandle(pages));
        pages=pages(indices);
        numeroreste=numeroini(indices);
        % ... avec l'identificateur de la diagraphie
        id=get(gcbf,'Userdata');
        idpages=get(pages,'Userdata');
        if iscell(idpages)
            indices=find([idpages{:}]==id);
        else
            indices=find(idpages==id);
        end
        pages=pages(indices);
        numeroreste=numeroreste(indices);
        % Aller page suivante
        numero=getappdata(gcbf,'numero');
        numero=find(numeroreste==numero);
        if numero~=length(pages)
            figure(pages(numero+1));
        end
    case 'last'
        % Parmi tous les pages cres,
        pages=getappdata(gcbf,'pages');
        % On ne retient que les pages existantes ...
        indices=find(ishandle(pages));
        pages=pages(indices);
        % ... et avec l'identificateur de la diagraphie
        id=get(gcbf,'Userdata');
        idpages=get(pages,'Userdata');
        if iscell(idpages)
            indices=find([idpages{:}]==id);
        else
            indices=find(idpages==id);
        end
        pages=pages(indices);
        % Aller 1e page
        figure(pages(length(pages)));
    otherwise
        return
    end
end

